part of dartscale;

/**
 * Represents a topic in the messagebus of the [Mediator], 
 * hosting a [StreamController](http://api.dartlang.org/docs/releases/latest/dart_async/StreamController.html) 
 * which can be used to receive messages send for this topic.
 */
class MediatorTopic {
  
  StreamController _streamController = new StreamController.broadcast(sync: true);
  
  StreamSubscription listen(void onData(dynamic event), 
                            {void onError(error), void onDone(), bool cancelOnError}){
    return this._streamController.stream.listen(onData, onError: onError, onDone: onDone, cancelOnError: cancelOnError);
  }
  
  add([dynamic event]) {
    this._streamController.add(event);
  }
}

/**
 * Represents a channel in the messagebus of the [Mediator], 
 * hosting a set of [MediatorTopic] which represent the topics
 * in this channel.
 */
class MediatorChannel {
  
  var _topics = new Map<String, MediatorTopic>();
  
  MediatorTopic topic(String topicName) {
    MediatorTopic topic = _topics[topicName];
    if (topic == null) {
      topic = new MediatorTopic();
      _topics[topicName] = topic;
    }
    
    return topic;
  }
}

/**
 * Implementes a messagebus, using Darts streaming API, with channels and topics.
 * It enables lously cuppled commuinications between all
 * different parts of the application.
 * 
 * Usually it's accessed over the [Sandbox] object and can
 * be used like this:
 * 
 *     this.sandbox.channel('mychannel').topic('mytopic').listen((data) {
 *         print('Hello ${data}!');
 *     });
 *     
 *     this.sandbox.channel('mychannel').topic('mytopic').add('World');
 */
class Mediator {
  
  var _channels = new Map<String, MediatorChannel>();
  
  MediatorChannel channel(String channelName) {
    var channel = _channels[channelName];
    if (channel == null) {
      channel = new MediatorChannel();
      _channels[channelName] = channel;
    }
    
    return channel;
  }
}